function [S,R] = cpmspec(p,h,g,fs,N)

% function [S,R] = cpmspec(p,h,g,fs,N)
% Calculates the power spectrum of a continuous phase modulation (CPM)
% digital signal with a random data stream. Both NRZ (unipolar) or 
% Bi-phase (bipolar) pulse shapes can be accommodated. For Bi-phase
% pulses the autocorrelation function R is divided into a continuous 
% part Rcon and a discrete part Rdis. The two spectra are calculated 
% separately. (see Aulin and Sundberg)
% Uses the algorithm described by Aulin, T and Sundberg,C-E in the paper
% "An easy way to calculate power spectra for digital FM"; IEE Proceedings,
% Vol.130, Part F, No. 6, Oct. 1983.
%
% p = column vector giving the probabilities of each symbol in a 
% set of size M.
% h = modulation index, such that the maximum phase shift over 1
% symbol period is (M-1)*h*pi.
% g = column vector of the samples of the frequency shaping pulse.
% fs = sampling rate, in samples per symbol.
% N = no. of points used in FFT to calculate the spectrum.
% R = autocorrelation function of the CPM signal.
% S = N point power spectrum with zero frequency at point N/2 + 1.
%--------------------------------------------------------------------
% Pulse shapes, g : (at fs samples per symbol period T)
%
% GSM => Gaussian filter, gf = Fc/fs*sqrt(2*pi/ln(2))*exp(-2*((pi*Fc*t).^2)/(ln(2)));
%        where Fc = 3dB frequency as fraction of bit rate (i.e. 0.3)
%    	       t = time at each sample, e.g. t=[-2:1/fs:2];
%        GSM pulse = square pulse of length T convolved with Gaussian filter
%                    i.e. g = conv(ones(fs,1),gf)
%
% Raised Cosine => g = sin(pi*t/T)/(pi*t/T)*cos(alpha*pi*t/T)/(1-(2*alpha*t/T)^2)
%                  where g(0)=1 and g(|t|=T/(2*alpha))=0.5*sin(pi/(2*alpha))
%                        T = 1, and t = [-2:1/fs:2], alpha = excess bandwidth factor
%



M = length(p);

% Normalise p.
p = p / sum(p);

% Calculate the phase pulse q from g and normalise so its final value = 0.5.

q = cumsum(g);
LT = length(q);
if q(LT) > 1e-3			% NRZ pulse
 q = q / (2 * q(LT));		% eqn(9)
else
 q = q / (2 * sum(abs(g)));	% Biphase pulse
end

% Calculate L and extend q to length L*T.
L = ceil(LT/fs);
%q = [0; q];
N = max(N,2*((L+1)*fs+1));

% Autocorrelation function R is first calculated from 0 to (L+1)*T, eqn(47)

j=sqrt(-1);
R=zeros(N,1);
R(1)=1;
expon=zeros(M,1);
k=[-(M-1):2:(M-1)]';
for l=1:M
  expon(l)=j*2*pi*h*k(l);
end


for m=0:L
  for td = 1:fs
    tdm = td + m*fs;	% eqn(48)  0 < tdm < (L+1)*T
    integral=0.0;
    for dt=1:fs		% integral from 0 to T
      prod_over_n=1.0;
      for n = (1-L):(m+1)	% product over n from n=(1-L) to m+1
        t1=dt+tdm-n*fs;
        if t1 <= 0		%eqn(10)
          q1 = 0.0;
        elseif t1 >= LT
          q1=q(LT);
        else
          q1=q(t1);
        end
             
        t2=dt-n*fs;
        if t2 <= 0
          q2 = 0.0;
        elseif t2 >= LT
          q2 = q(LT);
        else
          q2 = q(t2);
        end
           
        innersum=0;
        for l=1:M		% sum over k (odd) from k = -(M-1) to (M-1)
          innersum = innersum+p(l)*exp(expon(l)*(q1-q2));
        end
        prod_over_n = prod_over_n*innersum;          
      end
      integral=integral+prod_over_n;
    end
    R(tdm+1)=integral/fs;
  end
end

% Extend the correlation function to N/2 points

if q(LT) > 1e-3
  Calpha=0;
  for l=1:M
    Calpha=Calpha+p(l)*exp(expon(l)*q(LT));	%eqn(20) or (50)
  end
else
  Calpha = 1;					% Calpha = 1 if q(LT) = 0
end

% Extend the autocorrelation function for m > L 
% From eqn(21) only the values for tdm from L*T to (L+1)*T are needed

Rext=zeros(fs,1);
Rext=Calpha*R(L*fs+2:(L+1)*fs+1);

count=1; 
for t=(L+1)*fs+2: N/2+1
  R(t)=Rext(count);
  count=count+1;
  if count > fs
    Rext=Rext*Calpha;   
    count=1;
  end
end

if Calpha == 1
  Rcon = zeros(size(R));		% Rcon = 0 for tau > LT
  Rdis = R(L*fs+1:2*L*fs);
  Rcon(1:L*fs) = R(1:L*fs) - Rdis;	% subtract periodic component 
end					% from correlation function to 
    					% to get continuous part

% Extend the autocorrelation function so that it is conjugate symmetric around R(1)

if Calpha == 1
  for n=2:N/2
    Rcon(N-n+2)=conj(Rcon(n));		% extend only continuous part
  end
else
  for n=2:N/2
    R(N-n+2)=conj(R(n));
  end
end

% Calculate power spectrum using fft, zero frequency at S(N/2+1)

if Calpha ~= 1
  S=real(fftshift(fft(R/fs)));
else			% split the Autocorrelation function into its two parts
%  S=real(fftshift(fft(Rcon/fs)));
S = real(fft(Rcon/fs));			% Continuous spectrum from FFT
  for k = 0:fs-1
    Sdis(k+1,1) = 0;			% Discrete spectrum from Fourier series
    for n = 0:fs-1			% of one cycle of periodic part
      Sdis(k+1,1) = Sdis(k+1,1) + Rdis(n+1,1)*exp(j*2*pi*k*n/fs);
    end
  end
  Sdis = 1/fs*sqrt(Sdis.*conj(Sdis));
  specpos =1;
  count =1;
  specjump = N/fs;
  while count <= length(Sdis)		% Insert discrete spectral lines
    S(specpos,1) = S(specpos,1) + Sdis(count);
    count = count + 1;
    specpos = specpos + specjump;
  end
S = fftshift(S);
end
return;
