function h = latext(xpos,ypos,string,align,ptsize)

% function h = latext(xpos,ypos,string,align,ptsize)
%
% Plot the LaTeX-style text string on a figure, in ptsize point size
% and aligned to top/bottom, left/right or centre (default).
% e.g. align = 'tl' for top left, 'r' for centre right, ' ' for centre.
% sub/superscripts are preceded by '_' or '^'.
% Note {} are not supported so V_{max} must be written V_m_a_x.
% Greek letters / symbols are '\' followed by the equivalent
% letter in the symbol set; e.g. '\a' gives alpha.
% Similarly for math mode (italic) using '$'.
% Note: \f=phi, \h=eta, \j=varphi, \q=theta, \v=varpi, \w=omega, \y=psi.
% 
% Multiple effects must be in the order: _ or ^, $, \

% \\ produces new-line for multi-line text.
% \/ produces a new-line with a horiz line between the rows of text
% in order to produce fractions.
%
% The figure dimensions are adjusted to be the same as those
% used by 'print -deps' so that the text is the correct size relative
% to the graphics.
%
% A vector of handles to the separate text items is returned.

if nargin < 5, ptsize = 14; end
if nargin < 4, align = ' '; end

% Set figure dimensions.
% pos = [96    28   700   512];
% pos = [4    30   790   526]; % dimensions for figs under Win 95.
% if any(get(gcf,'position') ~= pos),
%   set(gcf,'position',pos);
% end

% Decode the string one symbol at a time and initially plot the result
% with its lower left at [xpos,ypos] to determine overall dimensions.
m = length(string);
h = [];
x = xpos;
y0 = ypos;
hgt = 0;
tothgt = 0;
small = round(ptsize * 0.75);

xmax = [];
row = [];
rowi = 1;
hline = [];

i = 1;
while i <= m,
  font = 'Helvetica';
  slant = 'normal';
  fs = ptsize;
  ch = string(i);
  y = y0;
  if string(i) == '_',   % Subscript
    y = y0 - 0.1*hgt;
    fs = small;
    i = i + 1;
    ch = string(i);
  elseif string(i) == '^',  % Subscript
    y = y0 + 0.4*hgt;
    fs = small;
    i = i + 1;
    ch = string(i);
  end
  if string(i) == '$',    % Math mode (Times italic)
    slant = 'italic';
   % font = 'Times';
    i = i + 1;
    ch = string(i);
  end
  if string(i) == '\',    % Greek / Symbol
    font = 'Symbol';
    i = i + 1;
    ch = string(i);
  end

% Correct for Times font looking smaller than symbol or Helvetica.
  if length(findstr(font,'Times')), fs = round(fs * 1.1); end

  if ch == '\',           % New line
    xmax = [xmax; x];
    y0 = y0 - 1.1*hgt;
    x = xpos;
    rowi = rowi + 1;
    tothgt = tothgt + 1.1*hgt;
  elseif (ch == '/') & length(findstr(font,'Symbol')), % Fraction.
    xmax = [xmax; x];
    hline = [hline; rowi y0-0.05*hgt]; % Row no. and vert position of line.
    y0 = y0 - 1.1*hgt;  % New baseline and x position.
    x = xpos;
    rowi = rowi + 1;
    tothgt = tothgt + 1.1*hgt;
  elseif ch == '|',  % TEXT cannot write '|' so do it as a line.
    hh = text(x,y,' ');
    row = [row; rowi];
    set(hh,'FontName',font,'FontSize',fs,'FontAngle',slant,...
      'vert','bottom','horiz','left');
    ext = get(hh,'Extent');
    x = x + ext(3);
    hgt = max(hgt,ext(4));
    hh = line([1;1]*(x-ext(3)/2),[0;ext(4)]+y,'color','k','Clipping','off');
    h = [h; hh];
  else                    % Write next character.
    hh = text(x,y,ch,'FontName',font,'FontSize',fs,'FontAngle',slant,...
      'vert','bottom','horiz','left');
    h = [h; hh];
    row = [row; rowi];
    ext = get(hh,'Extent');
    x = x + ext(3);
    hgt = max(hgt,ext(4));
  end

  i = i + 1;
end
xmax = [xmax; x];
tothgt = tothgt + hgt;

% Draw lines between rows if needed, of length = longer of rows
% above and below.
for i = 1:size(hline,1),
  rowi = hline(i,1);
  if xmax(rowi+1) > xmax(rowi), rowi = rowi + 1; end
  hh = line([xpos xmax(rowi)],[1 1]*hline(i,2),'color','k',...
   'Clipping','off');
  h = [h; hh];
  row = [row; rowi];  % Assign the line to the longer row.
end  

% Reposition text according to 'align'.
width = xmax - xpos;  % width is a vector of row widths.

dx = -0.5 * width;
dy = 0.5 * tothgt - hgt;

if length(findstr(align,'l')), dx = 0 * width; end
if length(findstr(align,'r')), dx = -width; end
if length(findstr(align,'t')), dy = -hgt; end
if length(findstr(align,'b')), dy = tothgt - hgt; end

for i = 1:size(h,1),
  if length(findstr(get(h(i),'Type'),'text')),
    xy = get(h(i),'position');
    xy(1) = xy(1) + dx(row(i));
    xy(2) = xy(2) + dy;
    set(h(i),'position',xy)
  else  % Line.
    x = get(h(i),'xdata') + dx(row(i));
    y = get(h(i),'ydata') + dy;
    set(h(i),'xdata',x,'ydata',y)
  end
end

% drawnow
return
